@@ -1,6 +1,14 @@
+"""
+Glitch robust MCMC search
+=========================
+
+MCMC search employing a signal hypothesis allowing for a glitch to
+be present in the data. The setup corresponds to a targeted search,
+and the simulated signal contains a single glitch.
+"""
+
 import numpy as np
 import pyfstat
-import gridcorner
 import time
 from PyFstat_example_make_data_for_search_on_1_glitch import (
     tstart,
@@ -17,7 +25,7 @@ from PyFstat_example_make_data_for_search_on_1_glitch import (
 )
 import os
 
-label = "semicoherent_glitch_robust_directed_MCMC_search_on_1_glitch"
+label = "PyFstat_example_glitch_robust_directed_MCMC_search_on_1_glitch"
 
 Nstar = 1000
 F0_width = np.sqrt(Nstar) * np.sqrt(12) / (np.pi * duration)
@@ -79,17 +87,16 @@ mcmc.run(save_loudest=False)  # uses CFSv2 which doesn't support glitch paramete
 dT = time.time() - t1
 mcmc.print_summary()
 
-fig_and_axes = gridcorner._get_fig_and_axes(4, 2, 0.05)
+print("Making corner plot...")
 mcmc.plot_corner(
     label_offset=0.25,
     truths={"F0": F0, "F1": F1, "delta_F0": delta_F0, "tglitch": tstart + dtglitch},
-    fig_and_axes=fig_and_axes,
     quantiles=(0.16, 0.84),
     hist_kwargs=dict(lw=1.5, zorder=-1),
     truth_color="C3",
 )
 
-mcmc.plot_cumulative_max()
+mcmc.plot_cumulative_max(savefig=True)
 
 print(("Prior widths =", F0_width, F1_width))
 print(("Actual run time = {}".format(dT)))
