@@ -1,16 +1,15 @@
+"""
+Directed grid search: Linear spindown
+==========================================
+
+Search for CW signal including one spindown parameter
+using a parameter space grid (i.e. no MCMC).
+"""
 import pyfstat
 import numpy as np
 import os
 
-try:
-    from gridcorner import gridcorner
-except ImportError:
-    raise ImportError(
-        "Python module 'gridcorner' not found, please install from "
-        "https://gitlab.aei.uni-hannover.de/GregAshton/gridcorner"
-    )
-
-label = os.path.splitext(os.path.basename(__file__))[0]
+label = "PyFstat_example_grid_search_F0F1"
 outdir = os.path.join("PyFstat_example_data", label)
 
 F0 = 30.0
@@ -77,20 +76,24 @@ search = pyfstat.GridSearch(
 )
 search.run()
 
+print("Plotting 2F(F0)...")
 search.plot_1D(xkey="F0", xlabel="freq [Hz]", ylabel="$2\\mathcal{F}$")
+print("Plotting 2F(F1)...")
 search.plot_1D(xkey="F1")
+print("Plotting 2F(F0,F1)...")
 search.plot_2D(xkey="F0", ykey="F1", colorbar=True)
 
-F0_vals = np.unique(search.data[:, 2]) - F0
-F1_vals = np.unique(search.data[:, 3]) - F1
-twoF = search.data[:, -1].reshape((len(F0_vals), len(F1_vals)))
+print("Making gridcorner plot...")
+F0_vals = np.unique(search.data["F0"]) - F0
+F1_vals = np.unique(search.data["F1"]) - F1
+twoF = search.data["twoF"].reshape((len(F0_vals), len(F1_vals)))
 xyz = [F0_vals, F1_vals]
 labels = [
     "$f - f_0$",
     "$\\dot{f} - \\dot{f}_0$",
     "$\\widetilde{2\\mathcal{F}}$",
 ]
-fig, axes = gridcorner(
+fig, axes = pyfstat.gridcorner(
     twoF, xyz, projection="log_mean", labels=labels, whspace=0.1, factor=1.8
 )
 fig.savefig(os.path.join(outdir, label + "_projection_matrix.png"))
