@@ -1,10 +1,18 @@
+"""
+Cumulative coherent 2F
+======================
+
+Compute the cumulative coherent F-statistic of a signal candidate.
+"""
+
+
 import os
 import numpy as np
 import pyfstat
 
 from pyfstat.helper_functions import get_predict_fstat_parameters_from_dict
 
-label = os.path.splitext(os.path.basename(__file__))[0]
+label = "PyFstat_example_twoF_cumulative"
 outdir = os.path.join("PyFstat_example_data", label)
 
 # Properties of the GW data
