@@ -50,7 +50,7 @@ class BaseSearchClass:
         return instance
 
     def _add_log_file(self, header=None):
-        """ Log output to a file, requires class to have outdir and label """
+        """Log output to a file, requires class to have outdir and label"""
         header = [] if header is None else header
         logfilename = os.path.join(self.outdir, self.label + ".log")
         with open(logfilename, "w") as fp:
@@ -66,7 +66,7 @@ class BaseSearchClass:
         logging.getLogger().addHandler(fh)
 
     def _get_list_of_matching_sfts(self):
-        """ Returns a list of sfts matching the attribute sftfilepattern """
+        """Returns a list of sfts matching the attribute sftfilepattern"""
         sftfilepatternlist = np.atleast_1d(self.sftfilepattern.split(";"))
         matches = [glob.glob(p) for p in sftfilepatternlist]
         matches = [item for sublist in matches for item in sublist]
@@ -99,7 +99,7 @@ class BaseSearchClass:
             self.sun_ephem = sun_ephem
 
     def _set_init_params_dict(self, argsdict):
-        """ Store the initial input arguments, e.g. for logging output. """
+        """Store the initial input arguments, e.g. for logging output."""
         argsdict.pop("self")
         self.init_params_dict = argsdict
 
@@ -257,6 +257,7 @@ class ComputeFstat(BaseSearchClass):
         maxStartTime=None,
         Tsft=1800,
         binary=False,
+        singleFstats=False,
         BSGL=False,
         transientWindowType=None,
         t0Band=None,
@@ -270,6 +271,7 @@ class ComputeFstat(BaseSearchClass):
         search_ranges=None,
         injectSources=None,
         injectSqrtSX=None,
+        randSeed=None,
         assumeSqrtSX=None,
         SSBprec=None,
         RngMedWindow=None,
@@ -278,6 +280,7 @@ class ComputeFstat(BaseSearchClass):
         computeAtoms=False,
         earth_ephem=None,
         sun_ephem=None,
+        allowedMismatchFromSFTLength=None,
     ):
         """
         Parameters
@@ -297,10 +300,13 @@ class ComputeFstat(BaseSearchClass):
             generated on the fly.
         binary : bool
             If true, search over binary parameters.
+        singleFstats : bool
+            If true, also compute the single-detector twoF values.
         BSGL : bool
             If true, compute the log10BSGL statistic rather than the twoF value.
             For details, see Keitel et al (PRD 89, 064023, 2014):
             https://arxiv.org/abs/1311.5738
+            Note this automatically sets `singleFstats=True` as well.
             Tuning parameters are currently hardcoded:
 
             * `Fstar0=15` for coherent searches.
@@ -360,6 +366,11 @@ class ComputeFstat(BaseSearchClass):
             List or comma-separated string: must match len(detectors)
             and/or the data in sftfilepattern.
             Detectors will be paired to list elements following alphabetical order.
+        randSeed : int or None
+            random seed for on-the-fly noise generation using `injectSqrtSX`.
+            Setting this to 0 or None is equivalent; both will randomise the seed,
+            following the behaviour of XLALAddGaussianNoise(),
+            while any number not equal to 0 will produce a reproducible noise realisation.
         assumeSqrtSX : float or list or str
             Don't estimate noise-floors but assume this (stationary) single-sided PSD.
             Single float or str value: use same for all IFOs.
@@ -392,6 +403,9 @@ class ComputeFstat(BaseSearchClass):
             Sun ephemeris file path.
             If None, will check standard sources as per
             helper_functions.get_ephemeris_files().
+        allowedMismatchFromSFTLength: float
+            Maximum allowed mismatch from SFTs being too long
+            [Default: what's hardcoded in XLALFstatMaximumSFTLength]
         """
 
         self._set_init_params_dict(locals())
@@ -399,6 +413,7 @@ class ComputeFstat(BaseSearchClass):
         self.init_computefstatistic()
         self.output_file_header = self.get_output_file_header()
         self.get_det_stat = self.get_fullycoherent_detstat
+        self.allowedMismatchFromSFTLength = allowedMismatchFromSFTLength
 
     def _get_SFTCatalog(self):
         """Load the SFTCatalog
@@ -541,7 +556,6 @@ class ComputeFstat(BaseSearchClass):
             self.whatToCompute += lalpulsar.FSTATQ_ATOMS_PER_DET
 
         FstatOAs = lalpulsar.FstatOptionalArgs()
-        FstatOAs.randSeed = lalpulsar.FstatOptionalArgsDefaults.randSeed
         if self.SSBprec:
             logging.info("Using SSBprec={}".format(self.SSBprec))
             FstatOAs.SSBprec = self.SSBprec
@@ -565,6 +579,8 @@ class ComputeFstat(BaseSearchClass):
             FstatOAs.assumeSqrtSX = mnf
         FstatOAs.prevInput = lalpulsar.FstatOptionalArgsDefaults.prevInput
         FstatOAs.collectTiming = lalpulsar.FstatOptionalArgsDefaults.collectTiming
+        if self.allowedMismatchFromSFTLength:
+            FstatOAs.allowedMismatchFromSFTLength = self.allowedMismatchFromSFTLength
 
         if hasattr(self, "injectSources") and type(self.injectSources) == dict:
             logging.info("Injecting source with params: {}".format(self.injectSources))
@@ -573,7 +589,6 @@ class ComputeFstat(BaseSearchClass):
             h0 = self.injectSources["h0"]
             cosi = self.injectSources["cosi"]
             use_aPlus = "aPlus" in dir(PP.Amp)
-            print("use_aPlus = {}".format(use_aPlus))
             if use_aPlus:  # lalsuite interface changed in aff93c45
                 PP.Amp.aPlus = 0.5 * h0 * (1.0 + cosi ** 2)
                 PP.Amp.aCross = h0 * cosi
@@ -620,6 +635,13 @@ class ComputeFstat(BaseSearchClass):
             ] = self.injectSqrtSX
         else:
             FstatOAs.injectSqrtSX = lalpulsar.FstatOptionalArgsDefaults.injectSqrtSX
+        # Here we are treating 0 and None as equivalent
+        # (use default, which is 0 and means "randomise the seed").
+        # See XLALAddGaussianNoise().
+        FstatOAs.randSeed = (
+            getattr(self, "randSeed", None)
+            or lalpulsar.FstatOptionalArgsDefaults.randSeed
+        )
         self._set_min_max_cover_freqs()
 
         logging.info("Initialising FstatInput")
@@ -643,6 +665,13 @@ class ComputeFstat(BaseSearchClass):
         logging.info("Initialising FstatResults")
         self.FstatResults = lalpulsar.FstatResults()
 
+        # always initialise the twoFX array,
+        # but only actually compute it if requested
+        self.twoFX = np.zeros(lalpulsar.PULSAR_MAX_DETECTORS)
+        self.singleFstats = self.singleFstats or self.BSGL  # BSGL implies twoFX
+        if self.singleFstats:
+            self.whatToCompute += lalpulsar.FSTATQ_2F_PER_DET
+
         if self.BSGL:
             if len(self.detector_names) < 2:
                 raise ValueError("Can't use BSGL with single detectors data")
@@ -657,24 +686,22 @@ class ComputeFstat(BaseSearchClass):
                 p_val_threshold = 1e-6
                 Fstar0s = np.linspace(0, 1000, 10000)
                 p_vals = scipy.special.gammaincc(2 * nsegs_eff, Fstar0s)
-                Fstar0 = Fstar0s[np.argmin(np.abs(p_vals - p_val_threshold))]
-                if Fstar0 == Fstar0s[-1]:
+                self.Fstar0 = Fstar0s[np.argmin(np.abs(p_vals - p_val_threshold))]
+                if self.Fstar0 == Fstar0s[-1]:
                     raise ValueError("Max Fstar0 exceeded")
             else:
-                Fstar0 = 15.0
-            logging.info("Using Fstar0 of {:1.2f}".format(Fstar0))
+                self.Fstar0 = 15.0
+            logging.info("Using Fstar0 of {:1.2f}".format(self.Fstar0))
             # assume uniform per-detector prior line-vs-Gaussian odds
-            oLGX = np.zeros(lalpulsar.PULSAR_MAX_DETECTORS)
-            oLGX[: self.numDetectors] = 1.0 / self.numDetectors
+            self.oLGX = np.zeros(lalpulsar.PULSAR_MAX_DETECTORS)
+            self.oLGX[: self.numDetectors] = 1.0 / self.numDetectors
             self.BSGLSetup = lalpulsar.CreateBSGLSetup(
                 numDetectors=self.numDetectors,
-                Fstar0sc=Fstar0,
-                oLGX=oLGX,
+                Fstar0sc=self.Fstar0,
+                oLGX=self.oLGX,
                 useLogCorrection=True,
                 numSegments=getattr(self, "nsegs", 1),
             )
-            self.twoFX = np.zeros(lalpulsar.PULSAR_MAX_DETECTORS)
-            self.whatToCompute += lalpulsar.FSTATQ_2F_PER_DET
 
         if self.transientWindowType:
             logging.info("Initialising transient parameters")
@@ -976,8 +1003,8 @@ class ComputeFstat(BaseSearchClass):
 
         These are also stored to `self.twoF` and `self.log10BSGL` respectively.
         As the basic statistic of this class, `self.twoF` is always computed.
-        If `self.BSGL`, additionally the single-detector 2F-stat values are saved
-        in `self.twoFX`.
+        If `self.singleFstats`, additionally the single-detector
+        2F-stat values are saved in `self.twoFX`.
 
         If transient parameters are enabled (`self.transientWindowType` is set),
         the full transient-F-stat map will also be computed here,
@@ -1006,9 +1033,10 @@ class ComputeFstat(BaseSearchClass):
             F0, F1, F2, Alpha, Delta, asini, period, ecc, tp, argp
         )
         if not self.transientWindowType:
-            if self.BSGL is False:
+            if self.singleFstats:
+                self.get_fullycoherent_single_IFO_twoFs()
+            if not self.BSGL:
                 return self.twoF
-            self.get_fullycoherent_single_IFO_twoFs()
             self.get_fullycoherent_log10BSGL()
             return self.log10BSGL
         self.get_transient_maxTwoFstat(tstart, tend)
@@ -1073,7 +1101,7 @@ class ComputeFstat(BaseSearchClass):
         )
         # We operate on a single frequency bin, so we grab the 0 component
         # of what is internally a twoF array.
-        self.twoF = np.float(self.FstatResults.twoF[0])
+        self.twoF = float(self.FstatResults.twoF[0])
         return self.twoF
 
     def get_fullycoherent_single_IFO_twoFs(self):
@@ -1087,10 +1115,16 @@ class ComputeFstat(BaseSearchClass):
             A list of the single-detector detection statistics twoF.
             Also stored as `self.twoFX`.
         """
-        self.twoFX[: self.FstatResults.numDetectors] = [
-            self.FstatResults.twoFPerDet(X)
-            for X in range(self.FstatResults.numDetectors)
-        ]
+        if not self.singleFstats:
+            raise RuntimeError(
+                "This function is available only if singleFstats or BSGL options were set."
+            )
+        self.twoFX[: self.FstatResults.numDetectors] = np.concatenate(
+            [
+                self.FstatResults.twoFPerDet(X)
+                for X in range(self.FstatResults.numDetectors)
+            ]
+        )
         return self.twoFX
 
     def get_fullycoherent_log10BSGL(self):
@@ -1256,6 +1290,8 @@ class ComputeFstat(BaseSearchClass):
         argp=None,
         tstart=None,
         tend=None,
+        transient_tstart=None,
+        transient_duration=None,
         num_segments=1000,
     ):
         """Calculate the cumulative twoF over subsets of the observation span.
@@ -1277,7 +1313,9 @@ class ComputeFstat(BaseSearchClass):
             If outside those: auto-truncated.
         num_segments: int
             Number of segments to split [tstart,tend] into.
-
+        transient_tstart, transient_duration: float or None
+            These are not actually used by this function,
+            but just included so a parameters dict can be safely passed.
         Returns
         -------
         cumulative_durations : ndarray of shape (num_segments,)
@@ -1451,7 +1489,7 @@ class ComputeFstat(BaseSearchClass):
         # Set up plot-related objects
         axis_kwargs = {
             "xlabel": f"Days from $t_\\mathrm{{start}}={actual_tstart_CFS:.0f}$",
-            "ylabel": "$\\log_{10}(\\mathrm{BSGL})_{\\mathrm{cumulative}$"
+            "ylabel": "$\\log_{10}(\\mathrm{BSGL})_{\\mathrm{cumulative}}$"
             if self.BSGL
             else "$\\widetilde{2\\mathcal{F}}_{\\mathrm{cumulative}}$",
             "xlim": (0, taus_CFS_days[-1]),
@@ -1504,6 +1542,20 @@ class ComputeFstat(BaseSearchClass):
                 alpha=0.2,
             )
 
+        if "transient_tstart" in CFS_input and "transient_duration" in CFS_input:
+            ax.axvspan(
+                (CFS_input["transient_tstart"] - actual_tstart_CFS) / 86400.0,
+                (
+                    CFS_input["transient_tstart"]
+                    + CFS_input["transient_duration"]
+                    - actual_tstart_CFS
+                )
+                / 86400.0,
+                color="lightgrey",
+                alpha=0.5,
+                label="transient duration",
+            )
+
         ax.legend(loc="best")
         if savefig:
             plt.tight_layout()
@@ -1570,6 +1622,7 @@ class SemiCoherentSearch(ComputeFstat):
         nsegs=None,
         sftfilepattern=None,
         binary=False,
+        singleFstats=False,
         BSGL=False,
         minStartTime=None,
         maxStartTime=None,
@@ -1579,11 +1632,14 @@ class SemiCoherentSearch(ComputeFstat):
         search_ranges=None,
         detectors=None,
         injectSources=None,
+        injectSqrtSX=None,
+        randSeed=None,
         assumeSqrtSX=None,
         SSBprec=None,
         RngMedWindow=None,
         earth_ephem=None,
         sun_ephem=None,
+        allowedMismatchFromSFTLength=None,
     ):
         """
         Only parameters with a special meaning for SemiCoherentSearch itself
@@ -1616,9 +1672,10 @@ class SemiCoherentSearch(ComputeFstat):
         self.computeAtoms = True  # for semicoh 2F from ComputeTransientFstatMap()
         self.tCWFstatMapVersion = "lal"
         self.cudaDeviceName = None
+        self.allowedMismatchFromSFTLength = allowedMismatchFromSFTLength
         self.init_computefstatistic()
         self.init_semicoherent_parameters()
-        if self.BSGL:
+        if self.singleFstats:
             self.twoFX_per_segment = np.zeros(
                 (lalpulsar.PULSAR_MAX_DETECTORS, self.nsegs)
             )
@@ -1716,8 +1773,8 @@ class SemiCoherentSearch(ComputeFstat):
         """Computes the detection statistic (twoF or log10BSGL) semi-coherently at a single point.
 
         As the basic statistic of this class, `self.twoF` is always computed.
-        If `self.BSGL`, additionally the single-detector 2F-stat values are saved
-        in `self.twoFX`.
+        If `self.singleFstats`, additionally the single-detector 2F-stat values
+        are saved in `self.twoFX` and (optionally) `self.twoFX_per_segment`.
 
         Parameters
         ----------
@@ -1727,7 +1784,7 @@ class SemiCoherentSearch(ComputeFstat):
             Optional: Binary parameters at which to compute the statistic.
         record_segments: boolean
             If True, store the per-segment F-stat values as `self.twoF_per_segment`
-            and (if `self.BSGL=True`) the per-detector per-segment F-stats
+            and (if `self.singleFstats`) the per-detector per-segment F-stats
             as `self.twoFX_per_segment`.
 
         Returns
@@ -1742,10 +1799,11 @@ class SemiCoherentSearch(ComputeFstat):
             F0, F1, F2, Alpha, Delta, asini, period, ecc, tp, argp, record_segments
         )
 
-        if self.BSGL is False:
+        if self.singleFstats:
+            self.get_semicoherent_single_IFO_twoFs(record_segments)
+        if not self.BSGL:
             return self.twoF
         else:
-            self.get_semicoherent_single_IFO_twoFs(record_segments)
             return self.get_semicoherent_log10BSGL()
 
     def get_semicoherent_twoF(
@@ -1814,6 +1872,8 @@ class SemiCoherentSearch(ComputeFstat):
         if record_segments:
             self.twoF_per_segment = twoF_per_segment
 
+        return self.twoF
+
     def get_semicoherent_single_IFO_twoFs(self, record_segments=False):
         """Computes the semi-coherent single-detector F-statss at a single point.
 
@@ -1832,6 +1892,10 @@ class SemiCoherentSearch(ComputeFstat):
             A list of the single-detector detection statistics twoF.
             Also stored as `self.twoFX`.
         """
+        if not self.singleFstats:
+            raise RuntimeError(
+                "This function is available only if singleFstats or BSGL options were set."
+            )
         for X in range(self.FstatResults.numDetectors):
             # For each detector, we need to build a MultiFstatAtomVector
             # because that's what the Fstat map function expects.
@@ -2029,6 +2093,7 @@ class SemiCoherentGlitchSearch(SearchForSignalWithJumps, ComputeFstat):
         nglitch=1,
         sftfilepattern=None,
         theta0_idx=0,
+        singleFstats=False,
         BSGL=False,
         minCoverFreq=None,
         maxCoverFreq=None,
@@ -2040,6 +2105,7 @@ class SemiCoherentGlitchSearch(SearchForSignalWithJumps, ComputeFstat):
         injectSources=None,
         earth_ephem=None,
         sun_ephem=None,
+        allowedMismatchFromSFTLength=None,
     ):
         """
         Only parameters with a special meaning for SemiCoherentGlitchSearch itself
