@@ -223,6 +223,18 @@ class Writer(BaseSearchClass):
     LAL routines would silently parse them wrongly.
     """
 
+    required_signal_parameters = [
+        # leaving out "F1","F2","psi","phi","tref" as they have defaults
+        "F0",
+        "Alpha",
+        "Delta",
+        "cosi",
+    ]
+    """List of parameters required for a successful execution of Makefakedata_v5.
+    The rest of available parameters are not required as they have default values
+    silently given by Makefakedata_v5
+    """
+
     @helper_functions.initializer
     def __init__(
         self,
@@ -253,6 +265,7 @@ class Writer(BaseSearchClass):
         transientStartTime=None,
         transientTau=None,
         randSeed=None,
+        timestampsFiles=None,
     ):
         """
         Parameters
@@ -261,8 +274,10 @@ class Writer(BaseSearchClass):
             A human-readable label to be used in naming the output files.
         tstart: int
             Starting GPS epoch of the data set.
+            NOTE: mutually exclusive with `timestampsFiles`.
         duration: int
             Duration (in GPS seconds) of the total data set.
+            NOTE: mutually exclusive with `timestampsFiles`.
         tref: float or None
             Reference time for simulated signals.
             Default is `None`, which sets the reference time to `tstart`.
@@ -291,6 +306,7 @@ class Writer(BaseSearchClass):
             Existing SFT files on top of which signals will be injected.
             If not `None`, additional constraints can be applied
             using the arguments `tstart` and `duration`.
+            NOTE: mutually exclusive with `timestampsFiles`.
         SFTWindowType: str or None
             LAL name of the windowing function to apply to the data.
         SFTWindowBeta: float
@@ -319,6 +335,14 @@ class Writer(BaseSearchClass):
         randSeed: int or None
             Optionally fix the random seed of Gaussian noise generation
             for reproducibility.
+        timestampsFiles: str or None
+            Comma-separated list of per-detector timestamps files
+            (simple text files, lines with <seconds> <nanoseconds>,
+            comments should use `%`, each time stamp gives the
+            start time of one SFT).
+            NOTE: mutually exclusive with [`tstart`,`duration`]
+            and with `noiseSFTs`.
+            WARNING: order must match that of `detectors`!
         """
 
         self.set_ephemeris_files(earth_ephem, sun_ephem)
@@ -446,6 +470,42 @@ class Writer(BaseSearchClass):
         self.duration = max(tend) - self.tstart
         self.detectors = ",".join(IFOs)
 
+    def _get_setup_from_timestampsFiles(self):
+        """
+        If timestampsFiles are given, use them to obtain relevant data parameters
+        (tstart, duration; but not detectors and Tsft as in the noiseSFTs case).
+        """
+        IFOs = self.detectors.split(",")
+        tsfiles = self.timestampsFiles.split(",")
+        if len(IFOs) != len(tsfiles):
+            raise ValueError(
+                f"Length of detectors=='{self.detectors}'"
+                f" does not match that of timestampsFiles=='{self.timestampsFiles}'"
+                f" ({len(IFOs)}!={len(tsfiles)})"
+            )
+        tstart = []
+        tend = []
+        self.sftfilenames = []  # This refers to the MFD output!
+        for X, IFO in enumerate(IFOs):
+            tsX = np.genfromtxt(tsfiles[X], comments="%")
+            this_start_time = int(tsX[0, 0])
+            this_end_time = int(tsX[-1, 0]) + self.Tsft
+            tstart.append(this_start_time)
+            tend.append(this_end_time)
+            self.sftfilenames.append(
+                lalpulsar.OfficialSFTFilename(
+                    IFO[0],
+                    IFO[1],
+                    len(tsX),
+                    self.Tsft,
+                    this_start_time,
+                    this_end_time - this_start_time,
+                    self.label,
+                )
+            )
+        self.tstart = min(tstart)
+        self.duration = max(tend) - self.tstart
+
     def _basic_setup(self):
         """Basic parameters handling, path setup etc."""
 
@@ -453,24 +513,33 @@ class Writer(BaseSearchClass):
         self.config_file_name = os.path.join(self.outdir, self.label + ".cff")
         self.theta = np.array([self.phi, self.F0, self.F1, self.F2])
 
-        required_signal_params = [
-            # leaving out "F1","F2","psi","phi","tref" as they have defaults
-            "F0",
-            "Alpha",
-            "Delta",
-            "cosi",
-        ]
         if self.h0 and np.any(
-            [getattr(self, k, None) is None for k in required_signal_params]
+            [getattr(self, k, None) is None for k in self.required_signal_parameters]
         ):
             raise ValueError(
                 "If h0>0, also need all of ({:s})".format(
-                    ",".join(required_signal_params)
+                    ",".join(self.required_signal_parameters)
                 )
             )
 
-        no_noiseSFTs_options = ["tstart", "duration", "detectors"]
-        if self.noiseSFTs is not None:
+        incompatible_with_TS = ["tstart", "duration", "noiseSFTs"]
+        TS_required_options = ["Tsft", "detectors"]
+        no_noiseSFTs_options = ["tstart", "duration", "Tsft", "detectors"]
+        if getattr(self, "timestampsFiles", None) is not None:
+            if np.any(
+                [getattr(self, k, None) is not None for k in incompatible_with_TS]
+            ):
+                raise ValueError(
+                    "timestampsFiles option is incompatible with"
+                    f" ({','.join(incompatible_with_TS)})."
+                )
+            if np.any([getattr(self, k, None) is None for k in TS_required_options]):
+                raise ValueError(
+                    "With timestampsFiles option, need also all of"
+                    f" ({','.join(TS_required_options)})."
+                )
+            self._get_setup_from_timestampsFiles()
+        elif self.noiseSFTs is not None:
             logging.warning(
                 "noiseSFTs is not None: Inferring tstart, duration, Tsft. "
                 "Input tstart and duration will be treated as SFT constraints "
@@ -478,9 +547,9 @@ class Writer(BaseSearchClass):
                 "internal consistency accross input SFTs."
             )
             self._get_setup_from_noiseSFTs()
-        elif np.any([getattr(self, k) is None for k in no_noiseSFTs_options]):
+        elif np.any([getattr(self, k, None) is None for k in no_noiseSFTs_options]):
             raise ValueError(
-                "Need either noiseSFTs or all of ({:s}).".format(
+                "Need either noiseSFTs, timestampsFiles or all of ({:s}).".format(
                     ",".join(no_noiseSFTs_options)
                 )
             )
@@ -643,36 +712,34 @@ class Writer(BaseSearchClass):
                 return False
         logging.info("...OK: file(s) found matching '{}'.".format(sftfile))
 
-        if "injectionSources" in cl_mfd:
-            if os.path.isfile(self.config_file_name):
-                if np.any(
-                    [
-                        os.path.getmtime(sftfile)
-                        < os.path.getmtime(self.config_file_name)
-                        for sftfile in self.sftfilenames
-                    ]
-                ):
-                    logging.info(
-                        (
-                            "...the config file '{}' has been modified since"
-                            " creation of the SFT file(s) '{}'. {}"
-                        ).format(self.config_file_name, self.sftfilepath, need_new)
-                    )
-                    return False
-                else:
-                    logging.info(
-                        "...OK: The config file '{}' is older than the SFT file(s)"
-                        " '{}'.".format(self.config_file_name, self.sftfilepath)
-                    )
-                    # NOTE: at this point we assume it's safe to re-use, since
-                    # _check_if_cff_file_needs_rewriting()
-                    # should have already been called before
-            else:
-                raise RuntimeError(
-                    "Commandline requires file '{}' but it is missing.".format(
-                        self.config_file_name
-                    )
+        if os.path.isfile(self.config_file_name):
+            if np.any(
+                [
+                    os.path.getmtime(sftfile) < os.path.getmtime(self.config_file_name)
+                    for sftfile in self.sftfilenames
+                ]
+            ):
+                logging.info(
+                    (
+                        "...the config file '{}' has been modified since"
+                        " creation of the SFT file(s) '{}'. {}"
+                    ).format(self.config_file_name, self.sftfilepath, need_new)
                 )
+                return False
+            else:
+                logging.info(
+                    "...OK: The config file '{}' is older than the SFT file(s)"
+                    " '{}'.".format(self.config_file_name, self.sftfilepath)
+                )
+                # NOTE: at this point we assume it's safe to re-use, since
+                # _check_if_cff_file_needs_rewriting()
+                # should have already been called before
+        elif "injectionSources" in cl_mfd:
+            raise RuntimeError(
+                "Commandline requires file '{}' but it is missing.".format(
+                    self.config_file_name
+                )
+            )
 
         logging.info("...checking new commandline against existing SFT header(s)...")
         # here we check one SFT header from each SFT file,
@@ -801,8 +868,11 @@ class Writer(BaseSearchClass):
         if self.SFTWindowType is not None:
             cl_mfd.append('--SFTWindowType="{}"'.format(self.SFTWindowType))
             cl_mfd.append("--SFTWindowBeta={}".format(self.SFTWindowBeta))
-        cl_mfd.append("--startTime={}".format(self.tstart))
-        cl_mfd.append("--duration={}".format(self.duration))
+        if getattr(self, "timestampsFiles", None) is not None:
+            cl_mfd.append("--timestampsFiles={}".format(self.timestampsFiles))
+        else:
+            cl_mfd.append("--startTime={}".format(self.tstart))
+            cl_mfd.append("--duration={}".format(self.duration))
         if hasattr(self, "fmin") and self.fmin:
             cl_mfd.append("--fmin={:.16g}".format(self.fmin))
         if hasattr(self, "Band") and self.Band:
@@ -897,6 +967,7 @@ class BinaryModulatedWriter(Writer):
         transientStartTime=None,
         transientTau=None,
         randSeed=None,
+        timestampsFiles=None,
     ):
         """
         Most parameters are the same as for the basic `Writer` class,
@@ -956,23 +1027,26 @@ class LineWriter(Writer):
     NOTE: This functionality is implemented via `lalapps_MakeFakeData_v4`'s `lineFeature` option.
     This version of MFD only supports one interferometer at a time.
 
-    NOTE: All signal parameters except for `h0` and `cosi` will be ignored.
-    `cosi` is used to rescale `h0` with the usual `sqrt(cosi**4 + 6*cosi**2 + 1)` relation.
+    NOTE: All signal parameters except for `h0`, `Freq`, `phi0` and transient parameters will be ignored.
     """
 
     mfd = "lalapps_Makefakedata_v4"
     """The executable (older version that supports the `--lineFeature` option)."""
 
-    required_mfd_line_parameters = [
-        "Freq",
-        "phi0",
+    required_signal_parameters = [
+        "F0",
+        "phi",
         "h0",
-        "cosi",
+    ]
+    """Required parameters for Makefakedata_v4 to success. Any other parameter is
+    silently given a default value by Makefakedata_v4.
+    """
+    signal_parameters_labels = required_signal_parameters + [
         "transientWindowType",
         "transientStartTime",
         "transientTau",
     ]
-    """Other signal parameters will be removed before passing to MFDv4."""
+    """Other signal parameters will be removed before passing to Makefakedata_v4."""
 
     def __init__(self, *args, **kwargs):
 
@@ -994,17 +1068,26 @@ class LineWriter(Writer):
         """
         super()._parse_args_consistent_with_mfd()
 
+        # FIXME: There should be a smoother way to translate keys
+        lal_required_signal_parameters = self.translate_keys_to_lal(
+            dict(
+                zip(
+                    self.required_signal_parameters,
+                    [0] * len(self.required_signal_parameters),
+                )
+            )
+        )
+
         if any(
-            key not in self.required_mfd_line_parameters
-            for key in self.signal_parameters
+            key not in lal_required_signal_parameters for key in self.signal_parameters
         ):
             logging.warning(
                 "Injection of line artifacts only uses the following parameters:\n"
-                f"{self.required_mfd_line_parameters}.\n"
+                f"{self.required_signal_parameters}.\n"
                 "Any other parameter will be purged from this class now"
             )
             params_to_purge = list(
-                set(self.signal_parameters) - set(self.required_mfd_line_parameters)
+                set(self.signal_parameters) - set(lal_required_signal_parameters)
             )
             logging.info(
                 "Purging input parameters that are not meaningful for LineWriter: {}".format(
@@ -1022,16 +1105,6 @@ class LineWriter(Writer):
                 "transientTau"
             )
 
-        if "cosi" in self.signal_parameters:
-            logging.info(
-                "Computing h0_eff from h0 and cosi. "
-                "This value will be stored in h0, "
-                "as cosi is *not* used by lineFeature."
-            )
-            eta = self.signal_parameters["cosi"]
-            eta2 = eta * eta
-            self.signal_parameters["h0"] *= np.sqrt(eta2 * eta2 + 6 * eta2 + 1)
-
     def calculate_fmin_Band(self):
         """Set fmin and Band for the output SFTs to cover.
 
@@ -1050,13 +1123,12 @@ class LineWriter(Writer):
     def _build_MFD_command_line(self):
         """Generate the SFT data calling lalapps_Makefakedata_v4."""
 
-        mfd = "lalapps_Makefakedata_v4"
-        cl_mfd = [mfd]
+        cl_mfd = [self.mfd]
 
         cl_mfd.append("--lineFeature=TRUE")
         cl_mfd.append("--outSingleSFT=TRUE")
         cl_mfd.append('--outSFTbname="{}"'.format(self.sftfilenames[0]))
-        cl_mfd.append("--IFO={}".format(self.detectors))
+        cl_mfd.append('--IFO="{}"'.format(self.detectors))
 
         if self.noiseSFTs is not None and self.SFTWindowType is None:
             raise ValueError(
@@ -1077,24 +1149,26 @@ class LineWriter(Writer):
                 )
             cl_mfd.append('--noiseSFTs="{}"'.format(self.noiseSFTs))
         if self.sqrtSX:
-            cl_mfd.append('--noiseSqrtSh="{}"'.format(self.sqrtSX))
+            cl_mfd.append("--noiseSqrtSh={}".format(self.sqrtSX))
 
         if self.SFTWindowType is not None:
             cl_mfd.append('--window="{}"'.format(self.SFTWindowType))
             cl_mfd.append("--tukeyBeta={}".format(self.SFTWindowBeta))
         cl_mfd.append("--startTime={}".format(self.tstart))
         cl_mfd.append("--duration={}".format(self.duration))
-        if hasattr(self, "fmin") and self.fmin:
+        if getattr(self, "fmin", None):
             cl_mfd.append("--fmin={:.16g}".format(self.fmin))
-        if hasattr(self, "Band") and self.Band:
+        if getattr(self, "Band", None):
             cl_mfd.append("--Band={:.16g}".format(self.Band))
         cl_mfd.append("--Tsft={}".format(self.Tsft))
         if self.h0:
             cl_mfd.append(
                 " ".join(
-                    f"--{key} {value}" for key, value in self.signal_parameters.items()
+                    f"--{key}={value:.16g}"
+                    for key, value in self.signal_parameters.items()
                 )
             )
+            cl_mfd.append("--cosi=0")  # Required by MFDv4
 
         earth_ephem = getattr(self, "earth_ephem", None)
         sun_ephem = getattr(self, "sun_ephem", None)
@@ -1144,6 +1218,7 @@ class GlitchWriter(SearchForSignalWithJumps, Writer):
         sun_ephem=None,
         transientWindowType="rect",
         randSeed=None,
+        timestampsFiles=None,
     ):
         """
         Most parameters are the same as for the basic `Writer` class,
@@ -1559,7 +1634,7 @@ class FrequencyModulatedArtifactWriter(Writer):
         # instead we use wildcards there.
         outfreq = int(np.floor(self.fmin))
         outwidth = int(np.floor(self.Band))
-        SFTFilename += f"_NB_F{outfreq:04d}Hz*_W{outwidth:04d}Hz*"
+        SFTFilename += f"_NBF{outfreq:04d}Hz*W{outwidth:04d}Hz*"
         SFTFilename += f"-{self.tstart}-{self.duration}.sft"
         SFTFile_fullpath = os.path.join(self.outdir, SFTFilename)
 
