@@ -45,7 +45,7 @@ default_Writer_params = {
     "Band": None,
 }
 
-default_signal_params = {
+default_signal_params_no_sky = {
     "F0": 30.0,
     "F1": -1e-10,
     "F2": 0,
@@ -53,8 +53,11 @@ default_signal_params = {
     "cosi": 0,
     "psi": 0,
     "phi": 0,
-    "Alpha": 5e-1,
-    "Delta": 1.2,
+}
+
+default_signal_params = {
+    **default_signal_params_no_sky,
+    **{"Alpha": 5e-1, "Delta": 1.2},
 }
 
 
@@ -204,7 +207,7 @@ class TestWriter(BaseForTestsWithData):
     label = "TestWriter"
     writer_class_to_test = pyfstat.Writer
     signal_parameters = default_signal_params
-    noiseSFT_detectors = "H1,L1"
+    multi_detectors = "H1,L1"  # this needs to be overwritable by child test classes that don't support multi-IFOs
 
     def test_make_cff(self):
         self.Writer.make_cff(verbose=True)
@@ -235,15 +238,18 @@ class TestWriter(BaseForTestsWithData):
             os.remove(self.Writer.config_file_name)
         if os.path.isfile(self.Writer.sftfilepath):
             os.remove(self.Writer.sftfilepath)
+
         # first run: make everything from scratch
         self.Writer.make_cff(verbose=True)
         self.Writer.run_makefakedata()
         time_first = os.path.getmtime(self.Writer.sftfilepath)
+
         # second run: should re-use .cff and .sft
         self.Writer.make_cff(verbose=True)
         self.Writer.run_makefakedata()
         time_second = os.path.getmtime(self.Writer.sftfilepath)
         self.assertTrue(time_first == time_second)
+
         # third run: touch the .cff to force regeneration
         time.sleep(1)  # make sure timestamp is actually different!
         os.system("touch {}".format(self.Writer.config_file_name))
@@ -251,6 +257,11 @@ class TestWriter(BaseForTestsWithData):
         time_third = os.path.getmtime(self.Writer.sftfilepath)
         self.assertFalse(time_first == time_third)
 
+        # fourth run: delete .cff and expect a RuntimeError
+        os.remove(self.Writer.config_file_name)
+        with pytest.raises(RuntimeError):
+            self.Writer.run_makefakedata()
+
     def test_noise_sfts(self):
         randSeed = 69420
 
@@ -261,7 +272,7 @@ class TestWriter(BaseForTestsWithData):
             duration=self.duration,
             Tsft=self.Tsft,
             tstart=self.tstart,
-            detectors=self.noiseSFT_detectors,
+            detectors=self.multi_detectors,
             randSeed=randSeed,
             SFTWindowType=self.SFTWindowType,
             SFTWindowBeta=self.SFTWindowBeta,
@@ -290,7 +301,7 @@ class TestWriter(BaseForTestsWithData):
             duration=self.duration,
             Tsft=self.Tsft,
             tstart=self.tstart,
-            detectors=self.noiseSFT_detectors,
+            detectors=self.multi_detectors,
             randSeed=randSeed,
             SFTWindowType=self.SFTWindowType,
             SFTWindowBeta=self.SFTWindowBeta,
@@ -371,7 +382,7 @@ class TestWriter(BaseForTestsWithData):
             )
         )
 
-    def test_data_with_gaps(self):
+    def test_noise_sfts_with_gaps(self):
         duration = 10 * self.Tsft
         gap_time = 4 * self.Tsft
         window = "tukey"
@@ -437,11 +448,74 @@ class TestWriter(BaseForTestsWithData):
         )
         self.assertTrue(os.path.isfile(expected_SFT_filepath))
 
+    def _test_writer_with_tsfiles(self, gaps=False):
+        """helper function to rerun with/without gaps"""
+        IFOs = self.multi_detectors.split(",")
+        tsfiles = [os.path.join(self.outdir, f"timestamps_{IFO}.txt") for IFO in IFOs]
+        numSFTs = []
+        for X, tsfile in enumerate(tsfiles):
+            with open(tsfile, "w") as fp:
+                k = 0
+                while k * self.Tsft < self.duration:
+                    if (
+                        not gaps or not k == X + 1
+                    ):  # add gaps at different points for each IFO
+                        fp.write(f"{self.tstart + k*self.Tsft} 0\n")
+                    k += 1
+            if gaps:
+                numSFTs.append(k - 1)
+            else:
+                numSFTs.append(k)
+            total_duration = k * self.Tsft
+        tsWriter = self.writer_class_to_test(
+            label="TestWriterWithTSFiles",
+            tref=self.tref,
+            Tsft=self.Tsft,
+            outdir=self.outdir,
+            sqrtSX=self.sqrtSX,
+            Band=self.Band,
+            detectors=self.multi_detectors,
+            SFTWindowType=self.SFTWindowType,
+            SFTWindowBeta=self.SFTWindowBeta,
+            randSeed=self.randSeed,
+            timestampsFiles=",".join(tsfiles),
+            **self.signal_parameters,
+        )
+        tsWriter.make_data(verbose=True)
+        for X, IFO in enumerate(IFOs):
+            expected_outfile = os.path.join(
+                tsWriter.outdir,
+                "{:1s}-{:d}_{:2s}_{:d}SFT_{:s}-{:d}-{:d}.sft".format(
+                    IFO[0],
+                    numSFTs[X],
+                    IFO,
+                    self.Tsft,
+                    tsWriter.label,
+                    self.tstart,
+                    total_duration,
+                ),
+            )
+            self.assertTrue(os.path.isfile(expected_outfile))
+            self.assertTrue(lalpulsar.ValidateSFTFile(expected_outfile) == 0)
+        if not gaps:
+            # test only first IFO's SFT against standard (tstart,duration) run
+            SFTnamesplit = tsWriter.sftfilepath.split(";")[0].split("Test")
+            self.assertTrue(self.Writer.sftfilepath.split("Test")[0] == SFTnamesplit[0])
+            self.assertTrue(
+                self.Writer.sftfilepath.split("Test")[1].split("-")[1:]
+                == SFTnamesplit[1].split("-")[1:]
+            )
+
+    def test_timestampsFiles(self):
+        self._test_writer_with_tsfiles(gaps=False)
+        self._test_writer_with_tsfiles(gaps=True)
+
 
 class TestLineWriter(TestWriter):
     label = "TestLineWriter"
     writer_class_to_test = pyfstat.make_sfts.LineWriter
-    noiseSFT_detectors = "H1"
+    signal_parameters = default_signal_params_no_sky
+    multi_detectors = "H1"
 
     def test_multi_ifo_fails(self):
         detectors = "H1,L1"
@@ -458,6 +532,30 @@ class TestLineWriter(TestWriter):
                 **self.signal_parameters,
             )
 
+    def test_makefakedata_usecached(self):
+
+        # Make everything from scratch
+        writer = self.writer_class_to_test(
+            outdir=self.outdir,
+            **default_Writer_params,
+            **default_signal_params,
+            **default_transient_params,
+        )
+        writer.make_data(verbose=True)
+        first_time = os.path.getmtime(writer.sftfilepath)
+
+        # Re-run, and should be unchanged
+        writer.make_data(verbose=True)
+        second_time = os.path.getmtime(writer.sftfilepath)
+        self.assertTrue(first_time == second_time)
+
+        # third run: touch the .cff to force regeneration
+        time.sleep(1)  # make sure timestamp is actually different!
+        os.system("touch {}".format(writer.config_file_name))
+        writer.run_makefakedata()
+        third_time = os.path.getmtime(writer.sftfilepath)
+        self.assertFalse(first_time == third_time)
+
     def _check_maximum_power_consistency(self, writer, return_line_power=False):
         times, freqs, data = pyfstat.helper_functions.get_sft_array(writer.sftfilepath)
         max_power_freq_index = np.argmax(data, axis=0)
@@ -502,35 +600,6 @@ class TestLineWriter(TestWriter):
 
         self._check_maximum_power_consistency(writer)
 
-    def test_cosi_scaling(self):
-        signal_params = default_signal_params.copy()
-        writer = self.writer_class_to_test(
-            outdir=self.outdir,
-            **default_Writer_params,
-            **signal_params,
-            **default_transient_params,
-        )
-
-        writer.make_data()
-        basic_power = self._check_maximum_power_consistency(
-            writer, return_line_power=True
-        )
-
-        signal_params["cosi"] = 1.0
-        writer = self.writer_class_to_test(
-            outdir=self.outdir,
-            **default_Writer_params,
-            **signal_params,
-            **default_transient_params,
-        )
-        writer.make_data()
-        corrected_power = self._check_maximum_power_consistency(
-            writer, return_line_power=True
-        )
-        self.assertTrue(
-            np.allclose(corrected_power / basic_power, np.sqrt(8), rtol=0, atol=1e-2)
-        )
-
 
 class TestWriterOtherTsft(TestWriter):
     label = "TestWriterOtherTsft"
@@ -740,6 +809,7 @@ class TestPredictFstat(BaseForTestsWithOutdir):
         self.assertAlmostEqual(twoF_sigma, chi2.std(df=4), places=5)
 
     def test_PFS_signal(self):
+        duration = 10 * default_Writer_params["duration"]
         twoF_expected, twoF_sigma = pyfstat.helper_functions.predict_fstat(
             h0=1,
             cosi=0,
@@ -747,15 +817,56 @@ class TestPredictFstat(BaseForTestsWithOutdir):
             Alpha=0,
             Delta=0,
             minStartTime=default_Writer_params["tstart"],
-            duration=default_Writer_params["duration"],
+            duration=duration,
+            IFOs=default_Writer_params["detectors"],
+            assumeSqrtSX=1,
+        )
+        print("predict_fstat() returned:" f" E[2F]={twoF_expected}+-{twoF_sigma}")
+        self.assertTrue(twoF_expected > 4)
+        self.assertTrue(twoF_sigma > 0)
+        # call again but this time using a dictionary of parameters
+        params = {
+            "h0": 1,
+            "cosi": 0,
+            "psi": 0,
+            "Alpha": 0,
+            "Delta": 0,
+            "F0": 0,
+            "F1": 0,
+        }
+        params = pyfstat.helper_functions.get_predict_fstat_parameters_from_dict(params)
+        twoF_expected_dict, twoF_sigma_dict = pyfstat.helper_functions.predict_fstat(
+            **params,
+            minStartTime=default_Writer_params["tstart"],
+            duration=duration,
             IFOs=default_Writer_params["detectors"],
             assumeSqrtSX=1,
         )
         print(
-            "predict_fstat() returned: E[2F]={}+-{}".format(twoF_expected, twoF_sigma)
+            "predict_fstat() called with a dict returned:"
+            f" E[2F]={twoF_expected_dict}+-{twoF_sigma_dict}"
         )
-        self.assertTrue(twoF_expected > 4)
-        self.assertTrue(twoF_sigma > 0)
+        self.assertEqual(twoF_expected_dict, twoF_expected)
+        # add transient parameters
+        params["transientWindowType"] = "rect"
+        params["transient_tstart"] = default_Writer_params["tstart"]
+        params["transient_duration"] = 0.5 * duration
+        params = pyfstat.helper_functions.get_predict_fstat_parameters_from_dict(params)
+        (
+            twoF_expected_transient,
+            twoF_sigma_transient,
+        ) = pyfstat.helper_functions.predict_fstat(
+            **params,
+            minStartTime=default_Writer_params["tstart"],
+            duration=duration,
+            IFOs=default_Writer_params["detectors"],
+            assumeSqrtSX=1,
+        )
+        print(
+            "predict_fstat() called with a dict including a transient returned:"
+            f" E[2F]={twoF_expected_transient}+-{twoF_sigma_transient}"
+        )
+        self.assertTrue(twoF_expected_transient < twoF_expected)
 
 
 class TestBaseSearchClass(unittest.TestCase):
@@ -842,6 +953,36 @@ class TestComputeFstat(BaseForTestsWithData):
         diff = np.abs(twoF2 - twoF) / twoF
         self.assertTrue(diff < 0.001)
 
+    def test_run_computefstatistic_allowedMismatchFromSFTLength(self):
+
+        long_Tsft_params = default_Writer_params.copy()
+        long_Tsft_params["Tsft"] = 3600
+        long_Tsft_params["duration"] = 4 * long_Tsft_params["Tsft"]
+        long_Tsft_params["label"] = "long_Tsft"
+        long_Tsft_params["F0"] = 1500
+        long_Tsft_params["Band"] = 2.0
+        long_Tsft_Writer = pyfstat.Writer(**long_Tsft_params)
+        long_Tsft_Writer.run_makefakedata()
+
+        search = pyfstat.ComputeFstat(
+            tref=long_Tsft_Writer.tref,
+            sftfilepattern=long_Tsft_Writer.sftfilepath,
+            minCoverFreq=1499.5,
+            maxCoverFreq=1500.5,
+            allowedMismatchFromSFTLength=0.1,
+        )
+        with pytest.raises(RuntimeError):
+            search.get_fullycoherent_twoF(F0=1500, F1=0, F2=0, Alpha=0, Delta=0)
+
+        search = pyfstat.ComputeFstat(
+            tref=long_Tsft_Writer.tref,
+            sftfilepattern=long_Tsft_Writer.sftfilepath,
+            minCoverFreq=1499.5,
+            maxCoverFreq=1500.5,
+            allowedMismatchFromSFTLength=0.5,
+        )
+        search.get_fullycoherent_twoF(F0=1500, F1=0, F2=0, Alpha=0, Delta=0)
+
     def test_run_computefstatistic_single_point_injectSources(self):
 
         predicted_FS = self.Writer.predict_fstat()
@@ -892,7 +1033,27 @@ class TestComputeFstat(BaseForTestsWithData):
 
     def test_get_fully_coherent_BSGL(self):
         # first pure noise, expect log10BSGL<0
-        search_H1L1 = pyfstat.ComputeFstat(
+        search_H1L1_noBSGL = pyfstat.ComputeFstat(
+            tref=self.tref,
+            minStartTime=self.tstart,
+            maxStartTime=self.tstart + self.duration,
+            detectors="H1,L1",
+            injectSqrtSX=np.repeat(self.sqrtSX, 2),
+            minCoverFreq=self.F0 - 0.1,
+            maxCoverFreq=self.F0 + 0.1,
+            BSGL=False,
+            singleFstats=True,
+            randSeed=self.randSeed,
+        )
+        twoF = search_H1L1_noBSGL.get_fullycoherent_detstat(
+            F0=self.F0,
+            F1=self.F1,
+            F2=self.F2,
+            Alpha=self.Alpha,
+            Delta=self.Delta,
+        )
+        twoFX = search_H1L1_noBSGL.get_fullycoherent_single_IFO_twoFs()
+        search_H1L1_BSGL = pyfstat.ComputeFstat(
             tref=self.tref,
             minStartTime=self.tstart,
             maxStartTime=self.tstart + self.duration,
@@ -901,8 +1062,9 @@ class TestComputeFstat(BaseForTestsWithData):
             minCoverFreq=self.F0 - 0.1,
             maxCoverFreq=self.F0 + 0.1,
             BSGL=True,
+            randSeed=self.randSeed,
         )
-        log10BSGL = search_H1L1.get_fullycoherent_detstat(
+        log10BSGL = search_H1L1_BSGL.get_fullycoherent_detstat(
             F0=self.F0,
             F1=self.F1,
             F2=self.F2,
@@ -910,8 +1072,41 @@ class TestComputeFstat(BaseForTestsWithData):
             Delta=self.Delta,
         )
         self.assertTrue(log10BSGL < 0)
+        self.assertTrue(
+            log10BSGL == lalpulsar.ComputeBSGL(twoF, twoFX, search_H1L1_BSGL.BSGLSetup)
+        )
         # now with an added signal, expect log10BSGL>0
-        search_H1L1 = pyfstat.ComputeFstat(
+        search_H1L1_noBSGL = pyfstat.ComputeFstat(
+            tref=self.tref,
+            minStartTime=self.tstart,
+            maxStartTime=self.tstart + self.duration,
+            detectors="H1,L1",
+            injectSqrtSX=np.repeat(self.sqrtSX, 2),
+            injectSources="{{Alpha={:g}; Delta={:g}; h0={:g}; cosi={:g}; Freq={:g}; f1dot={:g}; f2dot={:g}; refTime={:d};}}".format(
+                self.Alpha,
+                self.Delta,
+                self.h0,
+                self.cosi,
+                self.F0,
+                self.F1,
+                self.F2,
+                self.tref,
+            ),
+            minCoverFreq=self.F0 - 0.1,
+            maxCoverFreq=self.F0 + 0.1,
+            BSGL=False,
+            singleFstats=True,
+            randSeed=self.randSeed,
+        )
+        twoF = search_H1L1_noBSGL.get_fullycoherent_detstat(
+            F0=self.F0,
+            F1=self.F1,
+            F2=self.F2,
+            Alpha=self.Alpha,
+            Delta=self.Delta,
+        )
+        twoFX = search_H1L1_noBSGL.get_fullycoherent_single_IFO_twoFs()
+        search_H1L1_BSGL = pyfstat.ComputeFstat(
             tref=self.tref,
             minStartTime=self.tstart,
             maxStartTime=self.tstart + self.duration,
@@ -930,8 +1125,9 @@ class TestComputeFstat(BaseForTestsWithData):
             minCoverFreq=self.F0 - 0.1,
             maxCoverFreq=self.F0 + 0.1,
             BSGL=True,
+            randSeed=self.randSeed,
         )
-        log10BSGL = search_H1L1.get_fullycoherent_detstat(
+        log10BSGL = search_H1L1_BSGL.get_fullycoherent_detstat(
             F0=self.F0,
             F1=self.F1,
             F2=self.F2,
@@ -939,6 +1135,9 @@ class TestComputeFstat(BaseForTestsWithData):
             Delta=self.Delta,
         )
         self.assertTrue(log10BSGL > 0)
+        self.assertTrue(
+            log10BSGL == lalpulsar.ComputeBSGL(twoF, twoFX, search_H1L1_BSGL.BSGLSetup)
+        )
 
     def test_cumulative_twoF(self):
         Nsft = 100
@@ -974,7 +1173,9 @@ class TestComputeFstat(BaseForTestsWithData):
         print(
             "2F from get_fullycoherent_detstat() is {:.4f}"
             " while last value from calculate_twoF_cumulative() is {:.4f};"
-            " relative difference: {:g}".format(twoF, twoF_cumulative[-1], reldiff)
+            " relative difference: {:.2f}".format(
+                twoF, twoF_cumulative[-1], 100 * reldiff
+            )
         )
         self.assertTrue(reldiff < 0.1)
         idx = int(Nsft / 2)
@@ -985,15 +1186,47 @@ class TestComputeFstat(BaseForTestsWithData):
         print(
             "Middle 2F value from calculate_twoF_cumulative() is {:.4f}"
             " while from duration ratio we'd expect {:.4f}*{:.4f}={:.4f};"
-            " relative difference: {:g}".format(
+            " relative difference: {:.2f}%".format(
                 twoF_cumulative[idx],
                 taus[idx] / taus[-1],
                 twoF,
                 partial_2F_expected,
-                reldiff,
+                100 * reldiff,
             )
         )
         self.assertTrue(reldiff < 0.1)
+        _, _, pfs, pfs_sigma = search.predict_twoF_cumulative(
+            F0=self.Writer.F0,
+            Alpha=self.Writer.Alpha,
+            Delta=self.Writer.Delta,
+            h0=self.Writer.h0,
+            cosi=self.Writer.cosi,
+            psi=self.Writer.psi,
+            tstart=self.tstart,
+            tend=self.tstart + Nsft * self.Tsft,
+            IFOs=self.detectors,
+            assumeSqrtSX=self.sqrtSX,
+            num_segments=3,  # this is slow, so only do start,mid,end
+        )
+        reldiffmid = 100 * (twoF_cumulative[idx] - pfs[1]) / pfs[1]
+        reldiffend = 100 * (twoF_cumulative[-1] - pfs[2]) / pfs[2]
+        print(
+            "Predicted 2F values from predict_twoF_cumulative() are"
+            " {:.4f}+-{:.4f}(+-{:.2f}%) at midpoint of data"
+            " and {:.4f}+-{:.4f}(+-{:.2f}%) after full data,"
+            " , relative differences: {:.2f}% and {:.2f}%".format(
+                pfs[1],
+                pfs_sigma[1],
+                100 * pfs_sigma[1] / pfs[1],
+                pfs[2],
+                pfs_sigma[2],
+                100 * pfs_sigma[2] / pfs[2],
+                reldiffmid,
+                reldiffend,
+            )
+        )
+        self.assertTrue(reldiffmid < 0.25)
+        self.assertTrue(reldiffend < 0.25)
 
 
 class TestComputeFstatNoNoise(BaseForTestsWithData):
@@ -1162,19 +1395,31 @@ class TestSemiCoherentSearch(BaseForTestsWithData):
         )
         self.assertTrue(diff < 0.3)
 
-    def test_get_semicoherent_BSGL(self):
-
-        search = pyfstat.SemiCoherentSearch(
+    def _test_get_semicoherent_BSGL(self, **dataopts):
+        search_noBSGL = pyfstat.SemiCoherentSearch(
             label=self.label,
             outdir=self.outdir,
             nsegs=self.nsegs,
-            sftfilepattern=self.Writer.sftfilepath,
-            tref=self.Writer.tref,
-            search_ranges=self.search_ranges,
-            BSGL=True,
+            BSGL=False,
+            singleFstats=True,
+            **dataopts,
         )
-
-        BSGL = search.get_semicoherent_det_stat(
+        twoF = search_noBSGL.get_semicoherent_det_stat(
+            self.Writer.F0,
+            self.Writer.F1,
+            self.Writer.F2,
+            self.Writer.Alpha,
+            self.Writer.Delta,
+        )
+        twoFX = search_noBSGL.get_semicoherent_single_IFO_twoFs()
+        search_BSGL = pyfstat.SemiCoherentSearch(
+            label=self.label,
+            outdir=self.outdir,
+            nsegs=self.nsegs,
+            BSGL=True,
+            **dataopts,
+        )
+        log10BSGL = search_BSGL.get_semicoherent_det_stat(
             self.Writer.F0,
             self.Writer.F1,
             self.Writer.F2,
@@ -1182,7 +1427,68 @@ class TestSemiCoherentSearch(BaseForTestsWithData):
             self.Writer.Delta,
             record_segments=True,
         )
-        self.assertTrue(BSGL > 0)
+        self.assertTrue(log10BSGL > 0)
+        self.assertTrue(
+            log10BSGL == lalpulsar.ComputeBSGL(twoF, twoFX, search_BSGL.BSGLSetup)
+        )
+
+    def test_get_semicoherent_BSGL_SFTs(self):
+        dataopts = {
+            "sftfilepattern": self.Writer.sftfilepath,
+            "tref": self.Writer.tref,
+            "search_ranges": self.search_ranges,
+        }
+        self._test_get_semicoherent_BSGL(**dataopts)
+
+    def test_get_semicoherent_BSGL_inject(self):
+        dataopts = {
+            "tref": self.tref,
+            "minStartTime": self.tstart,
+            "maxStartTime": self.tstart + self.duration,
+            "detectors": "H1,L1",
+            "injectSqrtSX": np.repeat(self.sqrtSX, 2),
+            "minCoverFreq": self.F0 - 0.1,
+            "maxCoverFreq": self.F0 + 0.1,
+            "injectSources": self.Writer.config_file_name,
+            "randSeed": self.randSeed,
+        }
+        self._test_get_semicoherent_BSGL(**dataopts)
+
+    def test_get_semicoherent_twoF_allowedMismatchFromSFTLength(self):
+
+        long_Tsft_params = default_Writer_params.copy()
+        long_Tsft_params["Tsft"] = 3600
+        long_Tsft_params["duration"] = 4 * long_Tsft_params["Tsft"]
+        long_Tsft_params["label"] = "long_Tsft"
+        long_Tsft_params["F0"] = 1500
+        long_Tsft_params["Band"] = 2.0
+        long_Tsft_Writer = pyfstat.Writer(**long_Tsft_params)
+        long_Tsft_Writer.run_makefakedata()
+
+        search = pyfstat.SemiCoherentSearch(
+            label=self.label,
+            outdir=self.outdir,
+            tref=long_Tsft_Writer.tref,
+            sftfilepattern=long_Tsft_Writer.sftfilepath,
+            nsegs=self.nsegs,
+            minCoverFreq=1499.5,
+            maxCoverFreq=1500.5,
+            allowedMismatchFromSFTLength=0.1,
+        )
+        with pytest.raises(RuntimeError):
+            search.get_semicoherent_twoF(F0=1500, F1=0, F2=0, Alpha=0, Delta=0)
+
+        search = pyfstat.SemiCoherentSearch(
+            label=self.label,
+            outdir=self.outdir,
+            tref=long_Tsft_Writer.tref,
+            sftfilepattern=long_Tsft_Writer.sftfilepath,
+            nsegs=self.nsegs,
+            minCoverFreq=1499.5,
+            maxCoverFreq=1500.5,
+            allowedMismatchFromSFTLength=0.5,
+        )
+        search.get_semicoherent_twoF(F0=1500, F1=0, F2=0, Alpha=0, Delta=0)
 
 
 class TestSemiCoherentGlitchSearch(BaseForTestsWithData):
@@ -1379,6 +1685,12 @@ class BaseForMCMCSearchTests(BaseForTestsWithData):
             if assertTrue:
                 self.assertTrue(within)
 
+    def _test_plots(self):
+        self.search.plot_corner(add_prior=True)
+        self.search.plot_prior_posterior()
+        self.search.plot_cumulative_max()
+        self.search.plot_chainconsumer()
+
 
 class TestMCMCSearch(BaseForMCMCSearchTests):
     label = "TestMCMCSearch"
@@ -1480,6 +1792,7 @@ class TestMCMCSearch(BaseForMCMCSearchTests):
             self.search.print_summary()
             self._check_twoF_predicted()
             self._check_mcmc_quantiles()
+            self._test_plots()
 
 
 class TestMCMCSearchBSGL(TestMCMCSearch):
@@ -1547,6 +1860,7 @@ class TestMCMCSearchBSGL(TestMCMCSearch):
         maxTwoF_Fsearch = self.maxTwoF
         self._check_mcmc_quantiles(assertTrue=False)
         self.assertTrue(maxTwoF_Fsearch > self.twoF_predicted)
+        self._test_plots()
         # also run a BSGL search over the same data
         self.search = pyfstat.MCMCSearch(
             label=self.label + "-BSGL",
@@ -1576,6 +1890,7 @@ class TestMCMCSearchBSGL(TestMCMCSearch):
             np.abs(mode_F0_BSGLsearch - self.F0) < np.abs(mode_F0_Fsearch - self.F0)
         )
         self.assertTrue(maxTwoF_BSGLsearch < self.twoF_predicted)
+        self._test_plots()
 
 
 class TestMCMCSemiCoherentSearch(BaseForMCMCSearchTests):
@@ -1632,6 +1947,7 @@ class TestMCMCSemiCoherentSearch(BaseForMCMCSearchTests):
         self.assertTrue(np.abs(twoF_summed - twoF_sc) / twoF_sc < 0.01)
 
         self._check_mcmc_quantiles()
+        self._test_plots()
 
 
 class TestMCMCFollowUpSearch(BaseForMCMCSearchTests):
@@ -1679,6 +1995,7 @@ class TestMCMCFollowUpSearch(BaseForMCMCSearchTests):
         self.search.print_summary()
         self._check_twoF_predicted()
         self._check_mcmc_quantiles()
+        self._test_plots()
 
 
 class TestMCMCTransientSearch(BaseForMCMCSearchTests):
@@ -1745,6 +2062,7 @@ class TestMCMCTransientSearch(BaseForMCMCSearchTests):
         self.search.print_summary()
         self._check_twoF_predicted()
         self._check_mcmc_quantiles(transient=True)
+        self._test_plots()
 
     def test_transient_MCMC_tauonly(self):
 
@@ -1770,6 +2088,7 @@ class TestMCMCTransientSearch(BaseForMCMCSearchTests):
         self.search.print_summary()
         self._check_twoF_predicted()
         self._check_mcmc_quantiles(transient=True)
+        self._test_plots()
 
     def test_transient_MCMC_t0_tau(self):
 
@@ -1799,6 +2118,7 @@ class TestMCMCTransientSearch(BaseForMCMCSearchTests):
         self.search.print_summary()
         self._check_twoF_predicted()
         self._check_mcmc_quantiles(transient=True)
+        self._test_plots()
 
 
 class TestGridSearch(BaseForTestsWithData):
@@ -1811,9 +2131,33 @@ class TestGridSearch(BaseForTestsWithData):
     Band = 0.5
     BSGL = False
 
-    def test_grid_search(self):
-        search = pyfstat.GridSearch(
-            "grid_search",
+    def _test_plots(self, search_keys):
+        for key in search_keys:
+            self.search.plot_1D(xkey=key, savefig=True)
+        if len(search_keys) == 2:
+            self.search.plot_2D(xkey=search_keys[0], ykey=search_keys[1], colorbar=True)
+        vals = [
+            np.unique(self.search.data[key]) - getattr(self.Writer, key)
+            for key in search_keys
+        ]
+        twoF = self.search.data["twoF"].reshape([len(kval) for kval in vals])
+        corner_labels = [f"${key} - {key}_0$" for key in search_keys]
+        corner_labels.append("2F")
+        gridcorner_fig, gridcorner_axes = pyfstat.gridcorner(
+            twoF,
+            vals,
+            projection="log_mean",
+            labels=corner_labels,
+            whspace=0.1,
+            factor=1.8,
+        )
+        gridcorner_fig.savefig(
+            os.path.join(self.search.outdir, self.search.label + "_corner.png")
+        )
+
+    def test_grid_search_1D(self):
+        self.search = pyfstat.GridSearch(
+            "grid_search_F0",
             self.outdir,
             self.Writer.sftfilepath,
             F0s=self.F0s,
@@ -1824,13 +2168,35 @@ class TestGridSearch(BaseForTestsWithData):
             tref=self.tref,
             BSGL=self.BSGL,
         )
-        search.run()
-        self.assertTrue(os.path.isfile(search.out_file))
-        max2F_point = search.get_max_twoF()
-        self.assertTrue(np.all(max2F_point["twoF"] >= search.data["twoF"]))
+        self.search.run()
+        self.assertTrue(os.path.isfile(self.search.out_file))
+        max2F_point = self.search.get_max_twoF()
+        self.assertTrue(np.all(max2F_point["twoF"] >= self.search.data["twoF"]))
+        search_keys = ["F0"]  # only the ones that aren't 0-width
+        self._test_plots(search_keys)
+
+    def test_grid_search_2D(self):
+        self.search = pyfstat.GridSearch(
+            "grid_search_F0F1",
+            self.outdir,
+            self.Writer.sftfilepath,
+            F0s=self.F0s,
+            F1s=self.F1s,
+            F2s=[self.Writer.F2],
+            Alphas=[self.Writer.Alpha],
+            Deltas=[self.Writer.Delta],
+            tref=self.tref,
+            BSGL=self.BSGL,
+        )
+        self.search.run()
+        self.assertTrue(os.path.isfile(self.search.out_file))
+        max2F_point = self.search.get_max_twoF()
+        self.assertTrue(np.all(max2F_point["twoF"] >= self.search.data["twoF"]))
+        search_keys = ["F0", "F1"]  # only the ones that aren't 0-width
+        self._test_plots(search_keys)
 
     def test_grid_search_against_CFSv2(self):
-        search = pyfstat.GridSearch(
+        self.search = pyfstat.GridSearch(
             "grid_search",
             self.outdir,
             self.Writer.sftfilepath,
@@ -1841,10 +2207,10 @@ class TestGridSearch(BaseForTestsWithData):
             Deltas=[self.Writer.Delta],
             tref=self.tref,
         )
-        search.run()
-        self.assertTrue(os.path.isfile(search.out_file))
+        self.search.run()
+        self.assertTrue(os.path.isfile(self.search.out_file))
         pyfstat_out = pyfstat.helper_functions.read_txt_file_with_header(
-            search.out_file, comments="#"
+            self.search.out_file, comments="#"
         )
         CFSv2_out_file = os.path.join(self.outdir, "CFSv2_Fstat_out.txt")
         CFSv2_loudest_file = os.path.join(self.outdir, "CFSv2_Fstat_loudest.txt")
@@ -1885,7 +2251,7 @@ class TestGridSearch(BaseForTestsWithData):
 
     def test_semicoherent_grid_search(self):
         # FIXME this one doesn't check the results at all yet
-        search = pyfstat.GridSearch(
+        self.search = pyfstat.GridSearch(
             "sc_grid_search",
             self.outdir,
             self.Writer.sftfilepath,
@@ -1898,12 +2264,14 @@ class TestGridSearch(BaseForTestsWithData):
             nsegs=2,
             BSGL=self.BSGL,
         )
-        search.run()
-        self.assertTrue(os.path.isfile(search.out_file))
+        self.search.run()
+        self.assertTrue(os.path.isfile(self.search.out_file))
+        search_keys = ["F0"]  # only the ones that aren't 0-width
+        self._test_plots(search_keys)
 
     def test_glitch_grid_search(self):
-        search = pyfstat.GridGlitchSearch(
-            "grid_grid_search",
+        self.search = pyfstat.GridGlitchSearch(
+            "grid_glitch_search",
             self.outdir,
             self.Writer.sftfilepath,
             F0s=self.F0s,
@@ -1915,8 +2283,10 @@ class TestGridSearch(BaseForTestsWithData):
             tglitchs=[self.tref],
             # BSGL=self.BSGL,  # not supported by this class
         )
-        search.run()
-        self.assertTrue(os.path.isfile(search.out_file))
+        self.search.run()
+        self.assertTrue(os.path.isfile(self.search.out_file))
+        search_keys = ["F0", "F1"]  # only the ones that aren't 0-width
+        self._test_plots(search_keys)
 
 
 class TestGridSearchBSGL(TestGridSearch):
